#version 330 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;
layout (location = 2) out vec4 Depth;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
} fs_in;
flat in int lightNumber;
in float depthVS;

struct Light {
    vec3 Position;
    vec3 Color;
};

uniform Light lights[11];
uniform sampler2D colorTexture;
uniform vec3 viewPos;
uniform vec3 colorChange;
uniform float specularStrength;
uniform bool spaceship;

void main()
{           
    vec3 color = texture(colorTexture, fs_in.TexCoords).rgb + colorChange;
    vec3 normal = normalize(fs_in.Normal);
    // Ambient
    vec3 ambient = color;
    // Lighting
    vec3 diffLight = vec3(0.0f);
	vec3 specLight = vec3(0.0f);
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
	vec3 lightDir = normalize(lights[0].Position - fs_in.FragPos);
	vec3 reflectDir = reflect(-lightDir, normal); 

	//calculate diffuse for first(white) light
	float diff = max(dot(lightDir, normal), 0.0);
	vec3 diffuse = diff * lights[0].Color * color;
	diffLight += diffuse;

	//calculate specular light for first(white) light
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
	vec3 specular = specularStrength * spec * lights[0].Color; 
	specLight += specular;

	// blue Light
	lightDir = normalize(lights[1].Position - fs_in.FragPos);
	reflectDir = reflect(-lightDir, normal); 
	float theta = dot(lightDir, normalize(-vec3(0.0f, 0.0f, 1.0f)));

	if(theta > cos(0.2181661565)){
		//calculate diffuse for second(blue) light
		diff = max(dot(lightDir, normal), 0.0);
		vec3 result = lights[1].Color * diff * color;

		// Attenuation (use quadratic as we have gamma correction)
		float distance = length(fs_in.FragPos - lights[1].Position);
		result *= 1.0 / (distance * distance);
		diffLight += result;

		//calculate specular light for second(blue) light
		spec = pow(max(dot(viewDir, reflectDir), 0.0), 256);
		specular = specularStrength * spec * lights[1].Color; 
		specLight += specular;
	}

    for(int i = 2; i < lightNumber; i++)
    {
        // Diffuse
        vec3 lightDir = normalize(lights[i].Position - fs_in.FragPos);
        float diff = max(dot(lightDir, normal), 0.0);
        vec3 result = lights[i].Color * diff * color;
		      
        // Attenuation (use quadratic as we have gamma correction)
        float distance = length(fs_in.FragPos - lights[i].Position);
        result *= 1.0 / (distance * distance);
        diffLight += result;

		// Specular
		vec3 reflectDir = reflect(-lightDir, normal); 
		float spec = pow(max(dot(viewDir, reflectDir), 0.0), 256);
		vec3 specular = specularStrength * 0.01 * spec * lights[i].Color; 
		specLight += specular;
    }

    vec3 result = ambient + diffLight + specLight;

    // Check whether result is higher than some threshold, if so, output as bloom threshold color
    float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 1.0)
        BrightColor = vec4(result, 1.0);
    
    FragColor = vec4(result, 1.0f);
	//Depth = FragColor;
	if(spaceship) {
		//FragColor = vec4(1.0f, 0.0, 0.0, 1.0);
		Depth = vec4(1-depthVS, 1.0, 1.0, 1.0f);
	} else {
		Depth = vec4(1-depthVS, 0.0, 1.0, 1.0f);
	}
}