#version 330 core
// https://developer.nvidia.com/gpugems/GPUGems3/gpugems3_ch27.html

out vec4 FragColor;
in vec2 TexCoords;

uniform int numSamples;
uniform float strength;
uniform sampler2D depth;
uniform sampler2D scene;

uniform mat4 viewProjectInverse;
uniform mat4 prevViewProject;

void main()
{             
	float zOverW = texture(depth, TexCoords).r;
	vec4 H = vec4(TexCoords.x * 2 - 1, (1 - TexCoords.y) * 2 - 1, zOverW, 1);
	
	vec4 D = H * viewProjectInverse;
	vec4 worldPos = D / D.w;

	vec4 currentPos = H;
	vec4 previousPos = worldPos * prevViewProject;
	previousPos = previousPos / previousPos.w;
	vec4 tmp = (currentPos - previousPos)/2.f;
	previousPos.xy = previousPos.xy * 0.5 + 0.5;
	vec2 velocity = (previousPos.xy - TexCoords)*strength;

	vec4 result = texture(scene, TexCoords);
	vec2 texcoord = TexCoords;

	texcoord += velocity;
	for (int i = 1; i < numSamples; ++i, texcoord += velocity) {
		vec2 offset = velocity * (float(i) / float(numSamples - 1) - 0.5);
  		result += texture(scene, TexCoords + offset);
	}
 
	result /= float(numSamples);

	if(texture(depth, TexCoords).g == 1) {
		FragColor = texture(scene, TexCoords).rgba;
	} else {
		FragColor = result;
	}
}