#version 430 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
} fs_in;

struct Light {
    vec3 Position;
    vec3 Color;
};

uniform Light light;

uniform vec3 viewPos;
uniform vec3 colorChange;
uniform float specularStrength;

void main()
{           
    vec3 color = colorChange;
    vec3 normal = normalize(fs_in.Normal);
    // Ambient
    vec3 ambient = color;
    // Lighting
    vec3 diffLight = vec3(0.0f);
	vec3 specLight = vec3(0.0f);
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
	vec3 lightDir = normalize(light.Position - fs_in.FragPos);
	vec3 reflectDir = reflect(-lightDir, normal); 

	//calculate diffuse for first(white) light
	float diff = max(dot(lightDir, normal), 0.0);
	vec3 diffuse = diff * light.Color * color;
	diffLight += diffuse;

	//calculate specular light for first(white) light
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
	vec3 specular = specularStrength * spec * light.Color; 
	specLight += specular;

    vec3 result = ambient + diffLight + specLight;

    // Check whether result is higher than some threshold, if so, output as bloom threshold color
    float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 1.0)
        BrightColor = vec4(result, 1.0);
    
    FragColor = vec4(result, 1.0f);
	//FragColor = vec4(1.0f, 1.0f, 1.0f, 1.0f);
}